@@ -5,7 +5,7 @@
 #' process, the first is clustering values based on their ngram fingerprint (described here
 #' \url{https://github.com/OpenRefine/OpenRefine/wiki/Clustering-In-Depth}).
 #' The second step is merging values based on approximate string matching of
-#' the ngram fingerprints, using the [stringdistmatrix()] function from the
+#' the ngram fingerprints, using the [sd_lower_tri()] C function from the
 #' package \code{stringdist}.
 #'
 #' @param vect Character vector, items to be potentially clustered and merged.
@@ -26,15 +26,17 @@
 #'   c(d = 0.33, i = 0.33, s = 1, t = 0.5). This parameter gets passed along
 #'   to the \code{stringdist} function. Must be either
 #'   a numeric vector of length four, or NA.
-#' @param ... additional args to be passed along to \code{stringdist}.
+#' @param ... additional args to be passed along to the \code{stringdist}
+#'   function. The acceptable args are identical to those of
+#'   [stringdistmatrix()].
 #'
 #' @details The values of arg \code{weight} are edit distance values that
 #'  get passed to the \code{stringdist} edit distance function. The
 #'  param takes four arguments, each one is a specific type of edit, with
 #'  default penalty value.
 #'  \itemize{
-#'  \item d: Deletion, default value is 0.33
-#'  \item i: Insertion, default value is 0.33
+#'  \item d: deletion, default value is 0.33
+#'  \item i: insertion, default value is 0.33
 #'  \item s: substitution, default value is 1
 #'  \item t: transposition, default value is 0.5
 #'  }
@@ -67,38 +69,99 @@ n_gram_merge <- function(vect, numgram = 2, ignore_strings = NULL,
   stopifnot(is.numeric(edit_threshold) || is.na(edit_threshold))
   stopifnot(is.logical(bus_suffix))
   stopifnot(is.null(ignore_strings) || is.character(ignore_strings))
-  if (!is.na(weight) && !is.numeric(weight) && length(weight) != 4) {
-    stop("param 'weight' must be either a numeric vector with ",
-         "length four, or NA", call. = FALSE)
+  if (all(!is.na(weight))) {
+    if (!is.numeric(weight) && length(weight) != 4) {
+      stop("param 'weight' must be either a numeric vector with ",
+           "length four, or NA", call. = FALSE)
+    } else {
+      weight <- as.double(weight)
+    }
   }
   if ((!is.na(edit_threshold) && edit_threshold == 0) ||
       numgram == 1) {
     edit_threshold <- NA
   }
-  if (!is.na(edit_threshold) && is.na(weight)) {
+  edit_threshold_missing <- is.na(edit_threshold)
+  if (!edit_threshold_missing && is.na(weight)) {
     stop("param 'weight' must not be NA if 'edit_threshold'is not NA",
          call. = FALSE)
   }
 
   # If any args were passed via ellipsis, check to make sure they are valid
   # stringdist args.
-  ellip_args <- names(list(...))
-  if (any(c("a", "b") %in% ellip_args)) {
-    stop("'stringdistmatrix' args 'a' and 'b' cannot be set manually",
+  dots <- list(...)
+  dots_names <- names(dots)
+  if (any(c("a", "b") %in% dots_names)) {
+    stop("'stringdist' args 'a' and 'b' cannot be set manually",
          call. = FALSE)
   }
-  sd_args <- get("sd_args", envir = refinr_env)
-  if (!all(ellip_args %in% sd_args)) {
+  # Vector of valid arg names for stringdist.
+  sdm_args <- c("method", "useBytes", "weight", "q", "p", "bt", "useNames",
+                "nthread")
+  if (!all(dots_names %in% sdm_args)) {
     bad_args <- paste(
-      ellip_args[!ellip_args %in% sd_args],
+      dots_names[!dots_names %in% sdm_args],
       collapse = ", "
     )
     stop(paste("these input arg(s) are invalid:", bad_args), call. = FALSE)
   }
 
+  # More input validations for stringdist args.
+  if (!edit_threshold_missing) {
+    if (!"method" %in% dots_names) {
+      method <- 1L
+    } else {
+      sdm_methods <- c(osa = 0L, lv = 1L, dl = 2L, hamming = 3L, lcs = 4L,
+                       qgram = 5L, cosine = 6L, jaccard = 7L, jw = 8L,
+                       soundex = 9L)
+      if (!dots$method %in% names(sdm_methods)) {
+        stop(
+          sprintf("arg 'method' must be one of:\n%s",
+                  paste(names(sdm_methods), collapse = ", ")),
+          call. = FALSE
+        )
+      }
+      method <- sdm_methods[dots$method]
+    }
+
+    if (!"nthread" %in% dots_names) {
+      nthread <- getOption("sd_num_thread")
+    } else {
+      stopifnot(is.numeric(dots$nthread) && dots$nthread > 0)
+      nthread <- as.integer(dots$nthread)
+    }
+
+    if (!"useBytes" %in% dots_names) {
+      useBytes <- FALSE
+    } else {
+      stopifnot(is.logical(dots$useBytes))
+      useBytes <- dots$useBytes
+    }
+
+    if (!"q" %in% dots_names) {
+      q <- 1
+    } else {
+      stopifnot(dots$q >= 0)
+      q <- as.integer(dots$q)
+    }
+
+    if (!"p" %in% dots_names) {
+      p <- 0
+    } else {
+      stopifnot(dots$p <= 0.25 && dots$p >= 0)
+      p <- as.double(dots$p)
+    }
+
+    if (!"bt" %in% dots_names) {
+      bt <- 0
+    } else {
+      stopifnot(is.numeric(dots$bt))
+      bt <- as.double(dots$bt)
+    }
+  }
+
   # If approx string matching is being used, then get ngram == 1 keys for all
   # records.
-  edit_threshold_missing <- is.na(edit_threshold)
   univect <- cpp_unique(vect[!is.na(vect)])
   if (!edit_threshold_missing) {
     one_gram_keys <- get_fingerprint_ngram(univect, numgram = 1, bus_suffix,
@@ -116,22 +179,15 @@ n_gram_merge <- function(vect, numgram = 2, ignore_strings = NULL,
     return(ngram_merge_no_approx(n_gram_keys, univect, vect))
   }
 
-  # If approximate string matching is enabled, get initial clusters by finding
-  # all elements of n_gram_keys for which their associated one_gram_key has
-  # one or more matches within the entire list of one_gram_keys. Then create a
-  # stringdistmatrix for each cluster, which will be used to filter the
-  # initial clusters.
-
-  # Get initial clusters.
-  initial_clust <- get_ngram_initial_clusters(n_gram_keys, one_gram_keys)
-
-  # Create a stringdistmatrix for every element of initial_clust.
-  distmatrices <- lapply(initial_clust, function(x) {
-    as_matrix(stringdistmatrix(x, weight = weight, ...))
-  })
-
-  # Filter clusters based on distmatrices, then for each cluster, make mass
-  # edits to the values of vect related to that cluster.
-  return(ngram_merge_approx(n_gram_keys, univect, vect, distmatrices,
-                            edit_threshold, initial_clust))
+  # If approximate string matching is enabled, call ngram_merge_approx(). This
+  # will do the following:
+  # 1. Get initial clusters by finding all elements of n_gram_keys for which
+  #    their associated one_gram_key has one or more matches within the entire
+  #    list of one_gram_keys.
+  # 2. Create a stringdistmatrix for every initial cluster, then filter
+  #    clusters based on the dist matrices.
+  # 3. For each remaining cluster, make mass edits to the values of vect
+  #    related to that cluster. Return vect after mass edits have been made.
+  ngram_merge_approx(n_gram_keys, one_gram_keys, univect, vect, edit_threshold,
+                     method, weight, p, bt, q, useBytes, nthread)
 }
